﻿using Microsoft.Win32;
using System;
using System.Windows;

using TestLibrary;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            this.streamPlayerControl.StreamFailed  += streamPlayerControl_StreamHandler;
            this.streamPlayerControl.StreamStarted += streamPlayerControl_StreamHandler;
            this.streamPlayerControl.StreamStopped += streamPlayerControl_StreamHandler;
            this.playButton.Click                  += playButton_Click;
            this.stopButton.Click                  += stopButton_Click;
            this.imageButton.Click                 += imageButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 스트림 재생기 컨트롤 스트림 핸들러 - streamPlayerControl_StreamHandler(sender, e)

        /// <summary>
        /// 스트림 재생기 컨트롤 스트림 핸들러
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void streamPlayerControl_StreamHandler(object sender, RoutedEventArgs e)
        {
            SetButtonEnabled();

            if(e.RoutedEvent.Name == "StreamStarted")
            {
                this.statusLabel.Text = "재생";
            }
            else if(e.RoutedEvent.Name == "StreamFailed")
            {
                statusLabel.Text = "실패";

                MessageBox.Show
                (
                    ((StreamFailedEventArgs)e).Error,
                    "에러",
                    MessageBoxButton.OK,
                    MessageBoxImage.Error
                );
            }
            else if(e.RoutedEvent.Name == "StreamStopped")
            {
                statusLabel.Text = "중단";
            }
        }

        #endregion
        #region 재생하기 버튼 클릭시 처리하기 - playButton_Click(sender, e)

        /// <summary>
        /// 재생하기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void playButton_Click(object sender, RoutedEventArgs e)
        {
            Uri uri = new Uri(this.urlTextBox.Text);

            this.streamPlayerControl.StartPlay(uri);

            this.statusLabel.Text = "연결중...";
        }

        #endregion
        #region 중단하기 버튼 클릭시 처리하기 - stopButton_Click(sender, e)

        /// <summary>
        /// 중단하기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void stopButton_Click(object sender, RoutedEventArgs e)
        {
            this.streamPlayerControl.Stop();
        }

        #endregion
        #region 이미지 버튼 클릭시 처리하기 - stopButton_Click(sender, e)

        /// <summary>
        /// 이미지 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void imageButton_Click(object sender, RoutedEventArgs e)
        {
            SaveFileDialog dialog = new SaveFileDialog { Filter = "비트맵 이미지|*.bmp" };

            if(dialog.ShowDialog() == true)
            {
                this.streamPlayerControl.GetCurrentFrame().Save(dialog.FileName);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 버튼 이용 가능 여부 설정하기 - SetButtonEnabled()

        /// <summary>
        /// 버튼 이용 가능 여부 설정하기
        /// </summary>
        private void SetButtonEnabled()
        {
            this.playButton.IsEnabled  = !this.streamPlayerControl.IsPlaying; 
            this.stopButton.IsEnabled  = this.streamPlayerControl.IsPlaying;
            this.imageButton.IsEnabled = this.streamPlayerControl.IsPlaying;
        }

        #endregion
    }
}